﻿using LuxSciWebApiWrapper.Exceptions;
using LuxSciWebApiWrapper.LuxApiHttpClient;
using LuxSciWebApiWrapper.Models;
using LuxSciWebApiWrapper.Models.Authentication;
using System;
using System.Configuration;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

namespace LuxSciWebApiWrapper
{
    public class AuthApiWrapper
    {
        //members
        private static string apiSecretKey = string.Empty;
        private string apiToken = string.Empty;
        private string userName = string.Empty;
        private string password = string.Empty;

        static AuthApiWrapper()
        {
            //initialize secretkey
            apiSecretKey = ConfigurationManager.AppSettings["ApiSecretKey"];
        }
        public AuthApiWrapper()
        {
            //initialize members
            apiToken = ConfigurationManager.AppSettings["ApiToken"];
            userName = ConfigurationManager.AppSettings["UserName"];
            password = ConfigurationManager.AppSettings["Password"];
        }

        //Get user authentication token
        public async Task<string> GetAuthenticationToken()
        {
            //Populate userauthrequest using signature and epoch seconds
            var userRequest = new UserAuthRequest
            {
                User = userName,
                Password = password, 
                Token = apiToken, 
                Date = GetEpochSeconds(),
                Signature = GetUserHMACSignature() 
            };

            //call httpclient
            LuxSciResponse luxSciResponse = null;
            using (AuthHttpClient authHttpClient=new AuthHttpClient())
            {
                 luxSciResponse= await authHttpClient.GetAuthenticationCode(userRequest);
            }

            //throw error, if the success is not 1. otherwise, return authentication code
            if (luxSciResponse.success == 1)
            {
                return luxSciResponse.auth;
            }
            else
            {
                throw new ApiCallFailedException(luxSciResponse.error_message);
            }

        }

        //get account authentication token
        public async Task<string> GetAccountAuthenticationToken()
        {
            //populate accountauthrequest using signature and epoch seconds
            var userRequest = new AccountAuthRequest
            {
                
                Token = apiToken,
                Date = GetEpochSeconds(),
                Signature = GetAccountHMACSignature()
            };
            LuxSciResponse luxSciResponse = null;

            //call httpclient
            using (AuthHttpClient authHttpClient = new AuthHttpClient())
            {
                luxSciResponse = await authHttpClient.GetAuthenticationCode(userRequest);
            }

            //return authentication code, if success is 1
            if (luxSciResponse.success == 1)
            {
                return luxSciResponse.auth;
            }
            else
            {
                throw new ApiCallFailedException(luxSciResponse.error_message);
            }

        }

        //Get user signature
        private string GetUserHMACSignature()
        {
            return GetHMACSignature(apiToken + "\n" + GetEpochSeconds() + "\n" + userName + "\n" + password + "\n");
        }

        //Get account signature
        private string GetAccountHMACSignature()
        {
            return GetHMACSignature(apiToken + "\n" + GetEpochSeconds() + "\n" );
        }

        //Get HMACSignature for the given string using secret key
        public static string GetHMACSignature(string textToSign)
        {
            var hmac = new HMACSHA256(Encoding.UTF8.GetBytes(apiSecretKey));
            var hash = hmac.ComputeHash(Encoding.UTF8.GetBytes(textToSign));
            return BitConverter.ToString(hash).Replace("-", string.Empty);
        }

        //Get Epoch seconds
        private string GetEpochSeconds()
        {
            var epoch = DateTime.UtcNow - new DateTime(1970, 1, 1, 0, 0, 0);
            return ((long)epoch.TotalSeconds).ToString();
        }


        //Compute hash will be used by both user and account api
        public static string ComputeSha256Hash(string rawData)
        {
            // Create a SHA256   
            using (SHA256 sha256Hash = SHA256.Create())
            {
                // ComputeHash - returns byte array  
                byte[] bytes = sha256Hash.ComputeHash(Encoding.UTF8.GetBytes(rawData));

                // Convert byte array to a string   
                StringBuilder builder = new StringBuilder();
                for (int i = 0; i < bytes.Length; i++)
                {
                    builder.Append(bytes[i].ToString("x2"));
                }

                return builder.ToString();
            }
        }

        public static string ComputeSha256Hash(byte[] bytes)
        {
            // Create a SHA256   
            using (SHA256 sha256Hash = SHA256.Create())
            {
                return ToHex(sha256Hash.ComputeHash(bytes), false);
            }
        }

        private static string ToHex(byte[] bytes, bool upperCase)
        {
            StringBuilder result = new StringBuilder(bytes.Length * 2);

            for (int i = 0; i < bytes.Length; i++)
                result.Append(bytes[i].ToString(upperCase ? "X2" : "x2"));

            return result.ToString();
        }



    }


}
